@@ -15,6 +15,12 @@
 #' the Delta method (ignoring the covariance terms).
 #' @param zerotol Tolerance parameter for positivity checking in standardization. Default is zero. 
 #' The values of D <= zerotol * max(D, 0) are deemed to zero.
+#' @param expected Logical value defining the approximation of H_t in case of Gamma 
+#' and negative binomial distribution. Default is \code{FALSE} which matches the 
+#' algorithm of Durbin & Koopman (1997), whereas \code{TRUE} uses the expected value
+#' of observations in the equations, leading to results which match with \code{glm} (where applicable).
+#' The latter case was the default behaviour of KFAS before version 1.3.8.
+#' Essentially this is the difference between observed and expected information.
 #' @param ... Ignored.
 #' @examples
 #' modelNile <- SSModel(Nile ~ SSMtrend(1, Q = list(matrix(NA))), H = matrix(NA))
@@ -25,7 +31,7 @@
 #'   plot(modelNile)
 #' }
 
-plot.SSModel <- function(x, nsim = 0, zerotol = 0, ...) {
+plot.SSModel <- function(x, nsim = 0, zerotol = 0, expected = FALSE, ...) {
 
   op <- par(no.readonly = TRUE)
   on.exit(par(op))
@@ -70,7 +76,8 @@ plot.SSModel <- function(x, nsim = 0, zerotol = 0, ...) {
     }
 
   } else {
-    out <- KFS(x, smoothing = "mean", filtering = "mean", nsim = nsim)
+    out <- KFS(x, smoothing = "mean", filtering = "mean", nsim = nsim,
+      expected = expected)
     res <- rstandard(out, zerotol = zerotol)
 
     if (attr(x, "p") == 1) {
